# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import unohelper
import traceback
from collections import defaultdict
from operator import itemgetter
from itertools import groupby
from com.sun.star.awt import XDialogEventHandler
from com.sun.star.text.ControlCharacter import PARAGRAPH_BREAK
from com.sun.star.container import NoSuchElementException
from com.sun.star.beans import UnknownPropertyException

LBL_INSERT = "~INSERISCI"
LBL_UPDATE = "A~GGIORNA"
CONTENT_SUFFIX = "_index_content"
HEADER_SUFFIX = "_index_header"

class IndexCreator(unohelper.Base, XDialogEventHandler):
    def __init__(self, ctx, doc):
        self.ctx = ctx
        self.smgr = ctx.ServiceManager
        self.doc = doc
        self.viewcursor = doc.CurrentController.ViewCursor
        try:
            self.currentpos = self.viewcursor.Start
            self.vctext = self.viewcursor.Text
        except UnknownPropertyException:
            return
        self.undo = self.doc.UndoManager
        self.text = doc.Text
        self.sections = doc.TextSections
        self.indexes = defaultdict(lambda: defaultdict(set))
        self.indexdialog()

    def indexdialog(self):
        try:
            # need this for inserting index into table
            table = self.viewcursor.TextTable
            if table:
                self.viewcursor.gotoRange(table.Anchor, False)
            self.doc.lockControllers()
            dialogprovider = self.smgr.createInstanceWithArguments("com.sun.star.awt.DialogProvider2", (self.doc,))
            uri = "$(user)/Scripts/python/multi_lex_index.xml"
            dialog_url = self.substitute_variables(uri)
            dialog = dialogprovider.createDialogWithHandler(dialog_url, self)
            self.getindexes(self.text)
            self.mergepages()
            listbox = dialog.getControl("index")
            indexnames = list(self.indexes.keys())
            indexnames.sort()
            listbox.addItems(tuple(indexnames), 0)
            if indexnames:
                listbox.selectItemPos(0, True)
            dialog.execute()
        finally:
            self.doc.unlockControllers()

    def createindex(self, indexname, withheader=True):
        if not indexname:
            return
        try:
            self.undo.enterUndoContext('indice "{}" inserito'.format(indexname))
            if withheader:
                headersection = self.insertsection(indexname+HEADER_SUFFIX)
                headersection.Anchor.setString(indexname.capitalize())
                headersection.Anchor.ParaStyleName = "User Index Heading"
            contentsection = self.insertsection(indexname+CONTENT_SUFFIX)
            index = self.indexes[indexname]
            lines = list(index.values())
            lines.sort()
            content = '\n'.join(lines)
            contentsection.Anchor.setString(content)
            contentsection.Anchor.ParaStyleName = "User Index 1"
        except:
            traceback.print_exc()
        finally:
            self.undo.leaveUndoContext()

    def insertsection(self, name, pos=None):
        section = self.doc.createInstance("com.sun.star.text.TextSection")
        section.setName(name)
        if not pos:
            pos = self.currentpos
        self.vctext.insertTextContent(self.currentpos, section, False)
        return section

    def updateindex(self, indexname):
        if not indexname:
            return
        try:
            self.undo.enterUndoContext('indice "{}" aggiornato'.format(indexname))
            contentsection = self.sections.getByName(indexname+CONTENT_SUFFIX)
            self.viewcursor.gotoRange(contentsection.Anchor, False)
            index = self.indexes[indexname]
            lines = list(index.values())
            lines.sort()
            content = '\n'.join(lines)
            contentsection.Anchor.setString(content)
        except:
            traceback.print_exc()
        finally:
            self.undo.leaveUndoContext()

    def deleteindex(self, indexname):
        try:
            self.undo.enterUndoContext('indice "{}" cancellato'.format(indexname))
            for name in (indexname+HEADER_SUFFIX, indexname+CONTENT_SUFFIX):
                try:
                    section = self.sections.getByName(name)
                    anchor = section.Anchor
                    self.viewcursor.gotoRange(anchor, False)
                    anchor.setString("")
                    section.dispose()
                    anchor.Text.createEnumeration().nextElement().dispose()
                except NoSuchElementException:
                    pass
        finally:
            self.undo.leaveUndoContext()

    def getindexes(self, text):
        viewcursor = self.viewcursor
        paraenum = text.createEnumeration()
        while paraenum.hasMoreElements():
            para = paraenum.nextElement()
            try:
                portionenum = para.createEnumeration()
                while portionenum.hasMoreElements():
                    portion = portionenum.nextElement()
                    try:
                        mark = portion.DocumentIndexMark
                        if mark:
                            index = self.indexes[mark.UserIndexName]
                            if mark.MarkEntry:
                                viewcursor.gotoRange(portion, False)
                                index[mark.MarkEntry].add(viewcursor.Page)
                            else:
                                if portion.IsStart:
                                    viewcursor.gotoRange(portion.Start, False)
                                else:
                                    viewcursor.gotoRange(portion.Start, True)
                                    index[viewcursor.String.capitalize()].add(viewcursor.Page)
                    except AttributeError:
                        pass
            except AttributeError:
                pass

        # check sub texts
        if text == self.text:
            frames = self.doc.TextFrames
            for n in range(frames.Count):
                frame = frames.getByIndex(n)
                self.getindexes(frame)
            tables = self.doc.TextTables
            for n in range(tables.Count):
                table = tables.getByIndex(n)
                for cellname in table.CellNames:
                    cell = table.getCellByName(cellname)
                    self.getindexes(cell)

        viewcursor.gotoRange(self.currentpos, False)

    def mergepages(self):
        for index in self.indexes:
            indexmarks = self.indexes[index]
            for entry in indexmarks:
                pages = sorted(indexmarks[entry])
                pages = ', '.join(self.getpagerange(pages))
                indexmarks[entry] = "{} \t {}".format(entry, pages)


    def getpagerange(self, pages):
        data = []
        for k, g in groupby(enumerate(pages), lambda x: x[0]-x[1]):
            g = list(g)
            d = "{}".format(sorted(g)[0][1])
            if len(g) > 1:
                d += "-{}".format(list(g)[-1][1])
            data.append(d)
        return data

    def substitute_variables(self, uri):
        return self.smgr.createInstance("com.sun.star.util.PathSubstitution").substituteVariables(uri, True)

    def callHandlerMethod(self, dialog, event, method):
        try:
            if method == 'choice':
                choice = event.Source.SelectedItem + CONTENT_SUFFIX
                btnOK = dialog.getControl("btnOK")
                btnOK.setEnable(True)
                exists = isinstance(self.sections.ElementNames, tuple) and choice in self.sections.ElementNames
                if exists:
                    btnOK.setLabel(LBL_UPDATE)
                    dialog.getControl("btnDELETE").setEnable(True)
                else:
                    btnOK.setLabel(LBL_INSERT)
                    dialog.getControl("btnDELETE").setEnable(False)
            elif method == "OK":
                choice = dialog.getControl("index").SelectedItem
                if event.Source.Model.Label == LBL_INSERT:
                    self.createindex(choice)
                else:
                    self.updateindex(choice)
                dialog.endExecute()
            elif method == "delete":
                choice = dialog.getControl("index").SelectedItem
                self.deleteindex(choice)
                dialog.endExecute()
            else:
                return False
        except:
            traceback.print_exc()

    def getSupportedMethodNames(self):
        return 'choice',

def insertindex(event=None):
    ctx = XSCRIPTCONTEXT.getComponentContext()
    doc = XSCRIPTCONTEXT.getDocument()
    if doc.supportsService("com.sun.star.text.GenericTextDocument"):
        indexcreator = IndexCreator(ctx, doc)
